
/*
 *  ascp: a control panel for the AfterStep window manager
 *  Copyright (C) 1998  Nwanua Elumeze
 *
 *  You may distribute under the terms of the GNU General Public
 *  License as specified in the file, COPYING.
 *
 */


/* 
 * Naming Convention:
 *  
 * global variables:  GlobalVariables;
 * local variables:  localvariables;
 * application wide global variables:  APPWIDEGLOBVAR;
 *
 * local functions : regular_functions; 
 * application wide functions:   ascp_foobar;
 *
 */
				  
#include "xpm.h"


#include "ascp.h"


/* 
 * Global Variables used within within this file only.
 *
 * The philosophy is: "If a widget can keep a value,
 * there's no need to also have an element to also keep 
 * that value. The widget can be queried or assigned 
 * without writing additional callbacks.".
 */

/*----------------------------------*/
static FILE *IoFile;
/*----------------------------------*/

GtkWidget *EventList;
GtkWidget *SoundList;
GtkWidget *PlayButton;
GtkWidget *CurrentSound;
GtkWidget *AudioPlayCmd;
GtkSpinButton *AudioDelay;


static void parse_audio(gfloat, gfloat);
static gint play_sound(GtkWidget *, GdkEvent *, gpointer);



GtkWidget * ascp_draw_audio(gfloat min, gfloat max)
{
	/*      
	 * The toplevel table that shall be returned to
	 * populate_frame().
	 */ 
	GtkWidget *tabletop =  gtk_table_new(14, 14, TRUE);

	if(ASCPERROR)
		return gtk_label_new("ascp Error");

	/* 
	 * The two lists: "event" and "sound"; 
	 */
	{
		GtkWidget *frame;
		gchar *event[] = {"Event"};
		gchar *sound[] = {"Sound"};

		frame = gtk_frame_new(NULL);
		EventList = gtk_clist_new_with_titles(1, event);
		//gtk_clist_set_policy (GTK_CLIST(EventList), 
		//	GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
		gtk_container_add(GTK_CONTAINER(frame), EventList);
		gtk_table_attach_defaults( GTK_TABLE(tabletop), frame, 0, 5, 0, 10);

		frame = gtk_frame_new(NULL);
		SoundList = gtk_clist_new_with_titles(1, sound);
		//gtk_clist_set_policy (GTK_CLIST(SoundList), 
		//	GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
		gtk_container_add(GTK_CONTAINER(frame), SoundList);
		gtk_table_attach_defaults( GTK_TABLE(tabletop), frame, 9, 14, 0, 10);
	}


	/* 
	 * Entry for new sounds; and the two buttons to manipulate them
	 */
	{
		GtkWidget *frame;
		GtkWidget *button;

		frame =  gtk_frame_new(NULL);
		CurrentSound = gtk_entry_new_with_max_length(30);
		gtk_entry_set_text (GTK_ENTRY (CurrentSound), " ");
		gtk_container_add(GTK_CONTAINER(frame), CurrentSound);
		gtk_table_attach_defaults( GTK_TABLE(tabletop), frame, 9, 14, 12, 14);




		button = gtk_button_new_with_label("  New  ");
		gtk_table_attach( GTK_TABLE(tabletop), button, 9, 11, 10, 12,
			GTK_SHRINK, GTK_SHRINK, 0, 0);
		gtk_widget_set_sensitive(button, FALSE);

		button = gtk_button_new_with_label(" Delete ");
		gtk_table_attach( GTK_TABLE(tabletop), button, 12, 14, 10, 12,
			GTK_SHRINK, GTK_SHRINK, 0, 0);
		gtk_widget_set_sensitive(button, FALSE);

	}


	/* 
	 * Delay.
	 */
	{
		GtkWidget *label;
		GtkAdjustment *adj;
		GtkWidget *spinner;
		GtkWidget *subtable;


		subtable =  gtk_table_new(14, 14, TRUE);

		label = gtk_label_new("Timeout");
		gtk_table_attach_defaults(GTK_TABLE(tabletop), label, 6, 8, 0, 3); 


		adj =  (GtkAdjustment * ) gtk_adjustment_new (0, 1, 4, 1, 5, 0);
		spinner = gtk_spin_button_new (adj, 1, 0);
		AudioDelay = GTK_SPIN_BUTTON (spinner);
		gtk_spin_button_set_update_policy (AudioDelay, GTK_UPDATE_ALWAYS);
		gtk_table_attach_defaults(GTK_TABLE(subtable), spinner, 3, 14, 0, 2); 


		gtk_table_attach_defaults(GTK_TABLE(tabletop), subtable, 6, 8, 3, 5); 
	}
 

	/*
	 * The play command.
	 */
	{
		GtkWidget *frame;
		
		frame =  gtk_frame_new("Play Command");

		AudioPlayCmd = gtk_entry_new_with_max_length(30);
		gtk_entry_set_text (GTK_ENTRY (AudioPlayCmd), " ");
		gtk_container_add(GTK_CONTAINER(frame), AudioPlayCmd);
		gtk_table_attach_defaults( GTK_TABLE(tabletop), frame, 0, 8, 11, 14);

	}


	/*
	 * What _does_ the sound sound like?
	 */
	{
		GtkWidget *frame;

		frame =  gtk_frame_new(NULL);

		PlayButton = gtk_button_new_with_label("Play");
		/*
		gtk_signal_connect(GTK_OBJECT(PlayButton), "clicked",
				GTK_SIGNAL_FUNC(play_sound), NULL);
				*/
		gtk_container_border_width (GTK_CONTAINER (PlayButton), 2);
		gtk_container_add(GTK_CONTAINER(frame), PlayButton);
		gtk_table_attach_defaults(GTK_TABLE(tabletop), frame, 6, 8, 6, 8); 

		/* check if path is correct; if not error_popup */
		
	}

	parse_audio(min, max);


	gtk_widget_show_all(tabletop);
	return tabletop;
}

static gint play_sound(GtkWidget *blank, GdkEvent *event, gpointer alsoblank)
{
	if ( fork() == 0 )
	{
		gchar *player = g_malloc0(42);
				
		player = gtk_entry_get_text (GTK_ENTRY (AudioPlayCmd));
		if ( -1 == execl(player, "/usr/local/share/WindowMaker/Sounds/startup.wav"))
		{
			perror("ERROR> playing sound");
		}
		g_free(player);
		exit(EXIT_SUCCESS);

		return TRUE;
	}

	/* Meaningless in the context; just to shut the compiler up :-) */
	return FALSE;
}


static void parse_audio(gfloat min, gfloat max)
{
	gfloat incr;
	gchar *buffer = (gchar*) g_malloc0(1024);

	IoFile = ascp_open_file("audio", "r");

	incr = (max - min)/FILESIZE;

	while (fgets(buffer, 1024, IoFile) != NULL)
	{
		char* line = buffer;

		/*      
	 	 * Show our progress per total bytes per line read.
	 	 */
		ascp_update_status(NULL, NULL, min); 
		min = ( min >= max) ? max : min + incr*strlen(buffer); 

		for ( ; isspace(*line) ; line++);

		if( !ascp_strncasecmp(line, "*Audio", 6) )
		{
			line += 6;

			if( !ascp_strncasecmp(line, "PlayCmd", 7))
			{
				gchar *cmd = g_malloc0(24);

				sscanf(line + 7, "%s", cmd);
				gtk_entry_set_text (GTK_ENTRY (AudioPlayCmd), cmd);
				
				//g_print("command %s\n", cmd);
			}

		}
		

	}

	fclose(IoFile);
}


void ascp_write_audio()
{
	printf("write audio called\n");
}



