
/*
 *  ascp: a control panel for the AfterStep window manager
 *  Copyright (C) 1998  Nwanua Elumeze
 *
 *  You may distribute under the terms of the GNU General Public
 *  License as specified in the file, COPYING.
 *
 */


/* 
 * Naming Convention:
 *  
 * global variables:  GlobalVariables;
 * local variables:  localvariables;
 * application wide global variables:  APPWIDEGLOBVAR;
 *
 * local functions : regular_functions; 
 * application wide functions:   ascp_foobar;
 *
 */
				  
#include "xpm.h"
#include "ascp.h"


/* 
 * Global Variables used within within this file only.
 *
 * The philosophy is: "If a widget can keep a value,
 * there's no need to also have an element to also keep 
 * that value. The widget can be queried or assigned 
 * without writing additional callbacks.".
 */

/*----------------------------------*/
static FILE *IoFile;
/*----------------------------------*/


GtkWidget *Parent[2];
GtkWidget *Entries[3];

GtkWidget *InitList;
GtkWidget *RestartList;

GtkCList *CurrentList;

static void init_and_append_entry(gchar *, gint);
static void parse_autoexec(gfloat, gfloat);
static void init_restart_hide_show(GtkWidget *, gint);
static gint item_selected(GtkWidget *, gint, gint, GdkEventButton *, gpointer data);



GtkWidget * ascp_draw_autoexec(gfloat min, gfloat max)
{
	/*
	 * The toplevel table that shall be returned to
	 * populate_frame().
	 */
	GtkWidget *tabletop =  gtk_table_new(10, 14, TRUE);

	
	if(ASCPERROR)
		return gtk_label_new("ascp Error");

	/*
	 * Create the lists "Init" and "Restart".
	 */
	{
		gchar *titles[] = {"Action", "ID"}; 
		
		InitList = gtk_clist_new_with_titles(2, titles);
		/*callback */
		gtk_clist_set_column_width (GTK_CLIST(InitList), 0, 110);  
		gtk_clist_set_column_width (GTK_CLIST(InitList), 1, 20); 
		gtk_table_attach_defaults(GTK_TABLE(tabletop), InitList, 0, 7, 1, 8);
		//gtk_clist_set_policy (GTK_CLIST (InitList), 
		//	GTK_POLICY_ALWAYS, GTK_POLICY_AUTOMATIC);
		 gtk_signal_connect(GTK_OBJECT(InitList), "select_row",
			GTK_SIGNAL_FUNC(item_selected), NULL);

		  


		RestartList = gtk_clist_new_with_titles(2, titles);
		gtk_clist_set_column_width (GTK_CLIST(RestartList), 0, 110);  
		gtk_clist_set_column_width (GTK_CLIST(RestartList), 1, 20); 
		gtk_table_attach_defaults(GTK_TABLE(tabletop), RestartList, 0, 7, 1, 8);
		//gtk_clist_set_policy (GTK_CLIST (RestartList), 
		//	GTK_POLICY_ALWAYS, GTK_POLICY_AUTOMATIC);
		 gtk_signal_connect(GTK_OBJECT(RestartList), "select_row",
			GTK_SIGNAL_FUNC(item_selected), NULL);




		init_restart_hide_show(RestartList, ASCPCONFIG.initrestart);
		                    /* ^^^^^^^^^^^ */
		                      /* Not used */
	}



	/* 
	 * The option menu to choose from "Init" and "Restart".
	 */
	{
		GtkWidget *menu;
		GtkWidget *menuitem;
		GtkWidget *optionmenu;
		gint i=0;
		char *types [] = {
				"Init",
				"Restart"
				};


		menu = gtk_menu_new ();
		optionmenu = gtk_option_menu_new ();

		for(i=0; i < (sizeof(types)/sizeof(*types)); i++)
		{
			menuitem = gtk_menu_item_new_with_label(types[i]);
			gtk_menu_append(GTK_MENU (menu), menuitem);
			gtk_widget_show(menuitem);
			gtk_signal_connect_object(GTK_OBJECT(menuitem), "activate", 
				GTK_SIGNAL_FUNC(init_restart_hide_show) , (gpointer) i);
			
		}
		gtk_option_menu_set_menu (GTK_OPTION_MENU (optionmenu), menu);
		gtk_table_attach_defaults(GTK_TABLE(tabletop), optionmenu, 0, 3, 0, 1);
		gtk_widget_show_all(optionmenu);

	}


	/* 
	 * Create the three corresponding entries.
	 */
	{
		GtkWidget *frame;

		/* ID */
		frame = gtk_frame_new("ID");
		Entries[0] = gtk_entry_new_with_max_length (3);
		gtk_entry_set_text (GTK_ENTRY (Entries[0]), " ");
		gtk_container_add(GTK_CONTAINER(frame), Entries[0]);
		gtk_table_attach_defaults(GTK_TABLE(tabletop), frame, 12, 13, 1, 3);
		gtk_widget_show_all(frame);


		/* Action */
		frame = gtk_frame_new("Action");
		Entries[1] = gtk_entry_new_with_max_length (10);
		gtk_entry_set_text (GTK_ENTRY (Entries[1]), " ");
		gtk_container_add(GTK_CONTAINER(frame), Entries[1]);
		gtk_table_attach_defaults(GTK_TABLE(tabletop), frame, 8, 12, 1, 3);
		gtk_widget_show_all(frame);
		

		/* Command */
		frame = gtk_frame_new(NULL);
		//frame = gtk_frame_new("Command");
		Entries[2] = gtk_entry_new_with_max_length (60);
		gtk_entry_set_text (GTK_ENTRY (Entries[2]), " ");
		gtk_container_add(GTK_CONTAINER(frame), Entries[2]);
		gtk_table_attach_defaults(GTK_TABLE(tabletop), frame, 0 , 14, 8, 10);
		gtk_widget_show_all(frame);

	}


	/* 
	 * The ubiquitous, manipulative buttons.
	 * 
	 */
	{
		GtkWidget *hbox;
		GtkWidget *vbox;
		GtkWidget *button;


		vbox = gtk_vbox_new(TRUE, 0);
		hbox = gtk_hbox_new(TRUE, 0);

		button = gtk_button_new_with_label("New");
		/*
		gtk_table_attach( GTK_TABLE(tabletop), button, 9, 11, 4, 6,
			GTK_SHRINK, GTK_SHRINK, 0, 0);
			*/
		gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);

		gtk_widget_set_sensitive(button, FALSE);


		button = gtk_button_new_with_label("Change");
		/*
		gtk_table_attach( GTK_TABLE(tabletop), button, 11, 13, 4, 6,
			GTK_SHRINK, GTK_SHRINK, 0, 0);
		*/
		gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
		gtk_widget_set_sensitive(button, FALSE);

		button = gtk_button_new_with_label("  Delete  ");
		/*
		gtk_table_attach( GTK_TABLE(tabletop), button, 9, 13, 5, 6,
			GTK_SHRINK, GTK_SHRINK, 0, 0);
			*/
		gtk_widget_set_sensitive(button, FALSE);

		gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
		gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 0);


		gtk_widget_show_all(vbox);

		gtk_table_attach_defaults(GTK_TABLE(tabletop), vbox, 8, 13, 4, 7);



	}
	parse_autoexec(min, max);


	gtk_widget_show(tabletop);
	return tabletop;
}



static void parse_autoexec(gfloat min, gfloat max)
{
	gfloat incr;
	gint row=0;
	gchar *buffer = (gchar*) g_malloc0(1024);

	IoFile = ascp_open_file("autoexec", "r");

	incr = (max - min)/FILESIZE;

	CurrentList = GTK_CLIST(InitList);

	while (fgets(buffer, 1024, IoFile) != NULL)
	{
		char* line = buffer;

		/*      
	 	 * Show our progress per total bytes per line read.
	 	 */
		ascp_update_status(NULL, NULL, min); 
		min = ( min >= max) ? max : min + incr*strlen(buffer); 


		for ( ; isspace(*line) ; line++);

		if( !ascp_strncasecmp(line, "Function", 8) )
		{
			gchar *which = g_malloc(18);

			sscanf(line + 8, "%s", which);


			if( !ascp_strncasecmp(which, "\"InitFunction\"", 14) )
				CurrentList = GTK_CLIST(InitList);
			else if( !ascp_strncasecmp(which, "\"RestartFunction\"", 17) )
				CurrentList = GTK_CLIST(RestartList);

			g_free(which);
		}

		//g_print("the entries::%s\n", line);

		if( !ascp_strncasecmp(line, "Wait", 4) || 
			!ascp_strncasecmp(line, "Exec", 4) ||
			!ascp_strncasecmp(line, "Module", 6))
		{
			init_and_append_entry(line, row++);
		}


	}

	fclose(IoFile);
}

static void init_and_append_entry(gchar *line, gint row)
{
	gchar *action;
	gchar *id;
	gchar *command;
	gchar *tmp;
	int error = 0;


	tmp = g_malloc0(sizeof(line));

	for ( ; isspace(*line) ; line++)
		;
	
	for (tmp = line; *line!= '\0' && !isspace(*tmp) ; tmp++)
		;

	if(line == tmp)
		error = 1;

	if(!error)
	{
		action = g_malloc0(tmp - line + 1);
		strncpy(action, line,  tmp - line);
		action[tmp - line] = '\0';

		for (line = tmp; isspace(*line) ; line++)
			;

		for (tmp = line; *tmp != '\0' && !isspace(*tmp) ; tmp++)
			;

		if (line == tmp)
			error = 3;
	}

	if(!error)
	{
		id = g_malloc(tmp - line + 1);
		strncpy(id, line,  tmp - line);
		id[tmp - line] = '\0';

		for (line = tmp; isspace(*line) ; line++)
			;

		for (tmp = line; *tmp != '\0' && !isspace(*tmp) ; tmp++)
			;

		if (line == tmp)
			error = 4;
	}
	
	if (!error)
	{
		command = g_malloc(tmp - line + 1);
		strncpy(command, line,  tmp - line);
		command[tmp - line] = '\0';
	}

	{
		gchar *label[] = {action,id};

		gtk_clist_append (GTK_CLIST(CurrentList), label); 
		gtk_clist_set_row_data(GTK_CLIST(CurrentList), row, (gchar  *) command);

	}
}

static gint item_selected(GtkWidget   *clist, gint row, gint column,
				GdkEventButton *event, gpointer data)
{

		g_print("command %s\n", (gchar *)
				gtk_clist_get_row_data (GTK_CLIST (clist), row));

		/*
		gtk_entry_set_text (GTK_ENTRY (Entries[0]), "\"I\"");
		gtk_entry_set_text (GTK_ENTRY (Entries[1]), " ");
		*/

		return TRUE;
}
	


void ascp_write_autoexec()
{
	printf("write autoexec called\n");
}



static void init_restart_hide_show(GtkWidget *widget, gint which)
{
	if(which == 0)
	{
		gtk_widget_show(InitList);
		gtk_widget_hide(RestartList);
	}
	else
	{
		gtk_widget_show(RestartList);
		gtk_widget_hide(InitList);
	}
}

	
